@@ -36,6 +36,7 @@
 #include <QSettings>
 #include <QShortcut>
 #include <QShowEvent>
+#include <QStringList>
 #include <QStyleFactory>
 #include <cassert>
 #include <iostream>
@@ -57,6 +58,8 @@
 #include "Logger.h"
 #include "Misc.h"
 #include "ParametersCache.h"
+#include "PersistentMemory.h"
+#include "Settings.h"
 #include "Updater.h"
 #include "Utils.h"
 #include "Widgets/VisibleTagSelector.h"
@@ -159,19 +162,19 @@ MainWindow::MainWindow(QWidget * parent) : QMainWindow(parent), ui(new Ui::MainW
   auto searchAction = new QAction(this);
   searchAction->setShortcut(QKeySequence::Find);
   searchAction->setShortcutContext(Qt::ApplicationShortcut);
-  connect(searchAction, SIGNAL(triggered(bool)), ui->searchField, SLOT(setFocus()));
+  connect(searchAction, &QAction::triggered, ui->searchField, QOverload<>::of(&SearchFieldWidget::setFocus));
   addAction(searchAction);
 
   auto togglePreviewAction = new QAction(this);
   togglePreviewAction->setShortcut(QKeySequence("Ctrl+P"));
   togglePreviewAction->setShortcutContext(Qt::ApplicationShortcut);
-  connect(togglePreviewAction, SIGNAL(triggered(bool)), ui->cbPreview, SLOT(toggle()));
+  connect(togglePreviewAction, &QAction::triggered, ui->cbPreview, &QCheckBox::toggle);
   addAction(togglePreviewAction);
 
   searchAction = new QAction(this);
   searchAction->setShortcut(QKeySequence("/"));
   searchAction->setShortcutContext(Qt::ApplicationShortcut);
-  connect(searchAction, SIGNAL(triggered(bool)), ui->searchField, SLOT(setFocus()));
+  connect(searchAction, &QAction::triggered, ui->searchField, QOverload<>::of(&SearchFieldWidget::setFocus));
   addAction(searchAction);
 
   {
@@ -207,7 +210,7 @@ MainWindow::MainWindow(QWidget * parent) : QMainWindow(parent), ui(new Ui::MainW
   }
 
   QPalette p = QGuiApplication::palette();
-  DialogSettings::UnselectedFilterTextColor = p.color(QPalette::Disabled, QPalette::WindowText);
+  Settings::UnselectedFilterTextColor = p.color(QPalette::Disabled, QPalette::WindowText);
 
   _filtersPresenter = new FiltersPresenter(this);
   _filtersPresenter->setFiltersView(ui->filtersView);
@@ -224,7 +227,7 @@ MainWindow::MainWindow(QWidget * parent) : QMainWindow(parent), ui(new Ui::MainW
   QAction * escAction = new QAction(this);
   escAction->setShortcut(QKeySequence(Qt::Key_Escape));
   escAction->setShortcutContext(Qt::ApplicationShortcut);
-  connect(escAction, SIGNAL(triggered(bool)), this, SLOT(onEscapeKeyPressed()));
+  connect(escAction, &QAction::triggered, this, &MainWindow::onEscapeKeyPressed);
   addAction(escAction);
 
   CroppedImageListProxy::clear();
@@ -309,8 +312,8 @@ void MainWindow::setDarkTheme()
   qApp->setPalette(p);
 
   p = ui->cbInternetUpdate->palette();
-  p.setColor(QPalette::Text, DialogSettings::CheckBoxTextColor);
-  p.setColor(QPalette::Base, DialogSettings::CheckBoxBaseColor);
+  p.setColor(QPalette::Text, Settings::CheckBoxTextColor);
+  p.setColor(QPalette::Base, Settings::CheckBoxBaseColor);
   ui->cbInternetUpdate->setPalette(p);
   ui->cbPreview->setPalette(p);
 
@@ -331,7 +334,7 @@ void MainWindow::setDarkTheme()
   qApp->setStyleSheet(css);
   ui->inOutSelector->setDarkTheme();
   ui->vSplitterLine->setStyleSheet("QFrame{ border-top: 0px none #a0a0a0; border-bottom: 1px solid rgb(160,160,160);}");
-  DialogSettings::UnselectedFilterTextColor = DialogSettings::UnselectedFilterTextColor.darker(150);
+  Settings::UnselectedFilterTextColor = Settings::UnselectedFilterTextColor.darker(150);
 }
 #endif
 
@@ -470,6 +473,17 @@ void MainWindow::retrieveFilterAndParametersFromPluginParameters(QString & hash,
   }
 }
 
+QString MainWindow::screenGeometries()
+{
+  QList<QScreen *> screens = QGuiApplication::screens();
+  QStringList geometries;
+  for (QScreen * screen : screens) {
+    QRect geometry = screen->geometry();
+    geometries.push_back(QString("(%1,%2,%3,%4)").arg(geometry.x()).arg(geometry.y()).arg(geometry.width()).arg(geometry.height()));
+  }
+  return geometries.join(QString());
+}
+
 void MainWindow::onStartupFiltersUpdateFinished(int status)
 {
   bool ok = QObject::disconnect(Updater::getInstance(), &Updater::updateIsDone, this, &MainWindow::onStartupFiltersUpdateFinished);
@@ -477,7 +491,7 @@ void MainWindow::onStartupFiltersUpdateFinished(int status)
 
   ui->progressInfoWidget->stopAnimationAndHide();
   if (status == (int)Updater::UpdateStatus::SomeFailed) {
-    if (DialogSettings::notifyFailedStartupUpdate()) {
+    if (Settings::notifyFailedStartupUpdate()) {
       showMessage(tr("Filters update could not be achieved"), 3000);
     }
   } else if (status == (int)Updater::UpdateStatus::Successful) {
@@ -617,48 +631,36 @@ void MainWindow::showUpdateErrors()
 
 void MainWindow::makeConnections()
 {
-  connect(ui->zoomLevelSelector, SIGNAL(valueChanged(double)), ui->previewWidget, SLOT(setZoomLevel(double)));
+  connect(ui->zoomLevelSelector, &ZoomLevelSelector::valueChanged, ui->previewWidget, &PreviewWidget::setZoomLevel);
 
-  connect(ui->previewWidget, SIGNAL(zoomChanged(double)), this, SLOT(showZoomWarningIfNeeded()));
-  connect(ui->previewWidget, SIGNAL(zoomChanged(double)), this, SLOT(updateZoomLabel(double)));
-  connect(ui->previewWidget, SIGNAL(previewVisibleRectIsChanging()), &_processor, SLOT(cancel()));
-
-  connect(_filtersPresenter, SIGNAL(filterSelectionChanged()), this, SLOT(onFilterSelectionChanged()));
-
-  connect(ui->pbOk, SIGNAL(clicked(bool)), this, SLOT(onOkClicked()));
-  connect(ui->pbCancel, SIGNAL(clicked(bool)), this, SLOT(onCancelClicked()));
-  connect(ui->pbApply, SIGNAL(clicked(bool)), this, SLOT(onApplyClicked()));
-  connect(ui->tbResetParameters, SIGNAL(clicked(bool)), this, SLOT(onReset()));
+  connect(ui->previewWidget, &PreviewWidget::zoomChanged, this, &MainWindow::showZoomWarningIfNeeded);
+  connect(ui->previewWidget, &PreviewWidget::zoomChanged, this, &MainWindow::updateZoomLabel);
+  connect(ui->previewWidget, &PreviewWidget::previewVisibleRectIsChanging, &_processor, &GmicProcessor::cancel);
+  connect(_filtersPresenter, &FiltersPresenter::filterSelectionChanged, this, &MainWindow::onFilterSelectionChanged);
+  connect(ui->pbOk, &QPushButton::clicked, this, &MainWindow::onOkClicked);
+  connect(ui->pbCancel, &QPushButton::clicked, this, &MainWindow::onCancelClicked);
+  connect(ui->pbApply, &QPushButton::clicked, this, &MainWindow::onApplyClicked);
+  connect(ui->tbResetParameters, &QToolButton::clicked, this, &MainWindow::onReset);
   connect(ui->tbCopyCommand, &QToolButton::clicked, this, &MainWindow::onCopyGMICCommand);
-
-  connect(ui->tbUpdateFilters, SIGNAL(clicked(bool)), this, SLOT(onUpdateFiltersClicked()));
-
-  connect(ui->pbSettings, SIGNAL(clicked(bool)), this, SLOT(onSettingsClicked()));
-
-  connect(ui->pbFullscreen, SIGNAL(toggled(bool)), this, SLOT(onToggleFullScreen(bool)));
-
-  connect(ui->filterParams, SIGNAL(valueChanged()), this, SLOT(onParametersChanged()));
-  connect(ui->previewWidget, SIGNAL(previewUpdateRequested()), this, SLOT(onPreviewUpdateRequested()));
-  connect(ui->previewWidget, SIGNAL(keypointPositionsChanged(unsigned int, unsigned long)), this, SLOT(onPreviewKeypointsEvent(unsigned int, unsigned long)));
-
-  connect(ui->zoomLevelSelector, SIGNAL(zoomIn()), ui->previewWidget, SLOT(zoomIn()));
-  connect(ui->zoomLevelSelector, SIGNAL(zoomOut()), ui->previewWidget, SLOT(zoomOut()));
-  connect(ui->zoomLevelSelector, SIGNAL(zoomReset()), this, SLOT(onPreviewZoomReset()));
-
-  connect(ui->tbAddFave, SIGNAL(clicked(bool)), this, SLOT(onAddFave()));
+  connect(ui->tbUpdateFilters, &QToolButton::clicked, this, &MainWindow::onUpdateFiltersClicked);
+  connect(ui->pbSettings, &QPushButton::clicked, this, &MainWindow::onSettingsClicked);
+  connect(ui->pbFullscreen, &QPushButton::toggled, this, &MainWindow::onToggleFullScreen);
+  connect(ui->filterParams, &FilterParametersWidget::valueChanged, this, &MainWindow::onParametersChanged);
+  connect(ui->previewWidget, &PreviewWidget::previewUpdateRequested, this, QOverload<>::of(&MainWindow::onPreviewUpdateRequested));
+  connect(ui->previewWidget, &PreviewWidget::keypointPositionsChanged, this, &MainWindow::onPreviewKeypointsEvent);
+  connect(ui->zoomLevelSelector, &ZoomLevelSelector::zoomIn, ui->previewWidget, QOverload<>::of(&PreviewWidget::zoomIn));
+  connect(ui->zoomLevelSelector, &ZoomLevelSelector::zoomOut, ui->previewWidget, QOverload<>::of(&PreviewWidget::zoomOut));
+  connect(ui->zoomLevelSelector, &ZoomLevelSelector::zoomReset, this, &MainWindow::onPreviewZoomReset);
+  connect(ui->tbAddFave, &QToolButton::clicked, this, &MainWindow::onAddFave);
   connect(_filtersPresenter, &FiltersPresenter::faveAdditionRequested, this, &MainWindow::onAddFave);
   connect(ui->tbRemoveFave, &QToolButton::clicked, this, &MainWindow::onRemoveFave);
   connect(ui->tbRenameFave, &QToolButton::clicked, this, &MainWindow::onRenameFave);
-  connect(ui->inOutSelector, SIGNAL(inputModeChanged(InputMode)), this, SLOT(onInputModeChanged(InputMode)));
-
-  connect(ui->cbPreview, SIGNAL(toggled(bool)), this, SLOT(onPreviewCheckBoxToggled(bool)));
-  connect(ui->searchField, SIGNAL(textChanged(QString)), this, SLOT(search(QString)));
-
-  connect(ui->tbExpandCollapse, SIGNAL(clicked(bool)), this, SLOT(expandOrCollapseFolders()));
-  connect(ui->progressInfoWidget, SIGNAL(cancel()), this, SLOT(onProgressionWidgetCancelClicked()));
-
-  connect(ui->tbSelectionMode, SIGNAL(toggled(bool)), this, SLOT(onFiltersSelectionModeToggled(bool)));
-
+  connect(ui->inOutSelector, &InOutPanel::inputModeChanged, this, &MainWindow::onInputModeChanged);
+  connect(ui->cbPreview, &QCheckBox::toggled, this, &MainWindow::onPreviewCheckBoxToggled);
+  connect(ui->searchField, &SearchFieldWidget::textChanged, this, &MainWindow::search);
+  connect(ui->tbExpandCollapse, &QToolButton::clicked, this, &MainWindow::expandOrCollapseFolders);
+  connect(ui->progressInfoWidget, &ProgressInfoWidget::cancel, this, &MainWindow::onProgressionWidgetCancelClicked);
+  connect(ui->tbSelectionMode, &QToolButton::toggled, this, &MainWindow::onFiltersSelectionModeToggled);
   connect(&_processor, &GmicProcessor::previewImageAvailable, this, &MainWindow::onPreviewImageAvailable);
   connect(&_processor, &GmicProcessor::previewCommandFailed, this, &MainWindow::onPreviewError);
   connect(&_processor, &GmicProcessor::fullImageProcessingFailed, this, &MainWindow::onFullImageProcessingError);
@@ -694,12 +696,11 @@ void MainWindow::onPreviewUpdateRequested(bool synchronous)
   ui->previewWidget->normalizedVisibleRect(rect.x, rect.y, rect.w, rect.h);
 
   context.inputOutputState = ui->inOutSelector->state();
-  context.outputMessageMode = DialogSettings::outputMessageMode();
   ui->previewWidget->getPositionStringCorrection(context.positionStringCorrection.xFactor, context.positionStringCorrection.yFactor);
   context.zoomFactor = ui->previewWidget->currentZoomFactor();
   context.previewWindowWidth = ui->previewWidget->width();
   context.previewWindowHeight = ui->previewWidget->height();
-  context.previewTimeout = DialogSettings::previewTimeout();
+  context.previewTimeout = Settings::previewTimeout();
   // context.filterName = currentFilter.plainTextName; // Unused in this context
   // context.filterHash = currentFilter.hash; // Unused in this context
   context.filterCommand = currentFilter.previewCommand;
@@ -804,7 +805,6 @@ void MainWindow::processImage()
   GmicProcessor::FilterContext::VisibleRect & rect = context.visibleRect;
   rect.x = rect.y = rect.w = rect.h = -1;
   context.inputOutputState = ui->inOutSelector->state();
-  context.outputMessageMode = DialogSettings::outputMessageMode();
   context.filterName = currentFilter.plainTextName;
   context.filterFullPath = currentFilter.fullPath;
   context.filterHash = currentFilter.hash;
@@ -834,11 +834,34 @@ void MainWindow::onInputModeChanged(InputMode mode)
   ui->previewWidget->sendUpdateRequest();
 }
 
+void MainWindow::onVeryFirstShowEvent()
+{
+  adjustVerticalSplitter();
+  if (_newSession) {
+    Logger::clear();
+  }
+  QObject::connect(Updater::getInstance(), &Updater::updateIsDone, this, &MainWindow::onStartupFiltersUpdateFinished);
+  Logger::setMode(Settings::outputMessageMode());
+  Updater::setOutputMessageMode(Settings::outputMessageMode());
+  int ageLimit;
+  {
+    GMIC_SETTINGS(settings);
+    ageLimit = settings.value(INTERNET_UPDATE_PERIODICITY_KEY, INTERNET_DEFAULT_PERIODICITY).toInt();
+  }
+#ifndef _GMIC_QT_DISABLE_UPDATES_
+  const bool useNetwork = (ageLimit != INTERNET_NEVER_UPDATE_PERIODICITY);
+#else
+  const bool useNetwork = false;
+#endif
+  ui->progressInfoWidget->startFiltersUpdateAnimationAndShow();
+  Updater::getInstance()->startUpdate(ageLimit, 60, useNetwork);
+}
+
 void MainWindow::setZoomConstraint()
 {
   const FiltersPresenter::Filter & currentFilter = _filtersPresenter->currentFilter();
   ZoomConstraint constraint;
-  if (currentFilter.hash.isEmpty() || currentFilter.isAccurateIfZoomed || DialogSettings::previewZoomAlwaysEnabled() || (currentFilter.previewFactor == PreviewFactorAny)) {
+  if (currentFilter.hash.isEmpty() || currentFilter.isAccurateIfZoomed || Settings::previewZoomAlwaysEnabled() || (currentFilter.previewFactor == PreviewFactorAny)) {
     constraint = ZoomConstraint::Any;
   } else if (currentFilter.previewFactor == PreviewFactorActualSize) {
     constraint = ZoomConstraint::OneOrMore;
@@ -915,7 +938,7 @@ void MainWindow::onCancelClicked()
   if (_processor.isProcessing() && confirmAbortProcessingOnCloseRequest()) {
     if (_processor.isProcessing()) {
       _pendingActionAfterCurrentProcessing = ProcessingAction::Close;
-      connect(&_processor, SIGNAL(noMoreUnfinishedJobs()), this, SLOT(close()));
+      connect(&_processor, &GmicProcessor::noMoreUnfinishedJobs, this, &MainWindow::close);
       ui->progressInfoWidget->showBusyIndicator();
       ui->previewWidget->setOverlayMessage(tr("Waiting for cancelled jobs..."));
       _processor.cancel();
@@ -945,11 +968,13 @@ void MainWindow::onProgressionWidgetCancelClicked()
 void MainWindow::onReset()
 {
   if (!_filtersPresenter->currentFilter().hash.isEmpty() && _filtersPresenter->currentFilter().isAFave) {
+    PersistentMemory::clear();
     ui->filterParams->setVisibilityStates(_filtersPresenter->currentFilter().defaultVisibilityStates);
     ui->filterParams->setValues(_filtersPresenter->currentFilter().defaultParameterValues, true);
     return;
   }
   if (!_filtersPresenter->currentFilter().isNoPreviewFilter()) {
+    PersistentMemory::clear();
     ui->filterParams->reset(true);
   }
 }
@@ -1007,13 +1032,14 @@ void MainWindow::saveSettings()
 
   // Save all settings
 
-  DialogSettings::saveSettings(settings);
+  Settings::save(settings);
   settings.setValue("LastExecution/gmic_version", gmic_version);
   _processor.saveSettings(settings);
   settings.setValue("SelectedFilter", _filtersPresenter->currentFilter().hash);
   settings.setValue("Config/MainWindowPosition", frameGeometry().topLeft());
   settings.setValue("Config/MainWindowRect", rect());
   settings.setValue("Config/MainWindowMaximized", isMaximized());
+  settings.setValue("Config/ScreenGeometries", screenGeometries());
   settings.setValue("Config/PreviewEnabled", ui->cbPreview->isChecked());
   settings.setValue("LastExecution/ExitedNormally", true);
   settings.setValue("LastExecution/HostApplicationID", host_app_pid());
@@ -1048,21 +1074,25 @@ void MainWindow::loadSettings()
     setPreviewPosition(PreviewPosition::Left);
   }
 #ifndef _GMIC_QT_DISABLE_THEMING_
-  if (DialogSettings::darkThemeEnabled()) {
+  if (Settings::darkThemeEnabled()) {
     setDarkTheme();
   }
 #endif
-  if (!DialogSettings::logosAreVisible()) {
+  if (!Settings::visibleLogos()) {
     ui->logosLabel->hide();
   }
 
   // Mainwindow geometry
-  QPoint position = settings.value("Config/MainWindowPosition", QPoint()).toPoint();
+  QPoint position = settings.value("Config/MainWindowPosition", QPoint(20, 20)).toPoint();
   QRect r = settings.value("Config/MainWindowRect", QRect()).toRect();
+  const bool sameScreenGeometries = (settings.value("Config/ScreenGeometries", QString()).toString() == screenGeometries());
   if (settings.value("Config/MainWindowMaximized", false).toBool()) {
     ui->pbFullscreen->setChecked(true);
   } else {
-    if (r.isValid()) {
+    if (r.isValid() && sameScreenGeometries) {
+      if ((r.width() < 640) || (r.height() < 400)) {
+        r.setSize(QSize(640, 400));
+      }
       setGeometry(r);
       move(position);
     } else {
@@ -1242,6 +1272,7 @@ void MainWindow::activateFilter(bool resetZoom, const QList<QString> & values)
 
 void MainWindow::setNoFilter()
 {
+  PersistentMemory::clear();
   ui->filterParams->setNoFilter(_filtersPresenter->errorMessage());
   ui->previewWidget->disableRightClick();
   ui->previewWidget->setKeypoints(KeypointList());
@@ -1261,29 +1292,10 @@ void MainWindow::showEvent(QShowEvent * event)
 {
   TIMING;
   event->accept();
-  if (_showEventReceived) {
-    return;
+  if (!_showEventReceived) {
+    _showEventReceived = true;
+    onVeryFirstShowEvent();
   }
-  _showEventReceived = true;
-  adjustVerticalSplitter();
-  if (_newSession) {
-    Logger::clear();
-  }
-  QObject::connect(Updater::getInstance(), &Updater::updateIsDone, this, &MainWindow::onStartupFiltersUpdateFinished);
-  Logger::setMode(DialogSettings::outputMessageMode());
-  Updater::setOutputMessageMode(DialogSettings::outputMessageMode());
-  int ageLimit;
-  {
-    GMIC_SETTINGS(settings);
-    ageLimit = settings.value(INTERNET_UPDATE_PERIODICITY_KEY, INTERNET_DEFAULT_PERIODICITY).toInt();
-  }
-#ifndef _GMIC_QT_DISABLE_UPDATES_
-  const bool useNetwork = (ageLimit != INTERNET_NEVER_UPDATE_PERIODICITY);
-#else
-  const bool useNetwork = false;
-#endif
-  ui->progressInfoWidget->startFiltersUpdateAnimationAndShow();
-  Updater::getInstance()->startUpdate(ageLimit, 60, useNetwork);
 }
 
 void MainWindow::resizeEvent(QResizeEvent * e)
@@ -1301,10 +1313,10 @@ bool MainWindow::askUserForGTKFavesImport()
   messageBox.setDefaultButton(QMessageBox::Yes);
   QCheckBox * cb = new QCheckBox(tr("Don't ask again"));
 #ifndef _GMIC_QT_DISABLE_THEMING_
-  if (DialogSettings::darkThemeEnabled()) {
+  if (Settings::darkThemeEnabled()) {
     QPalette p = cb->palette();
-    p.setColor(QPalette::Text, DialogSettings::CheckBoxTextColor);
-    p.setColor(QPalette::Base, DialogSettings::CheckBoxBaseColor);
+    p.setColor(QPalette::Text, Settings::CheckBoxTextColor);
+    p.setColor(QPalette::Base, Settings::CheckBoxBaseColor);
     cb->setPalette(p);
   }
 #endif
@@ -1366,8 +1378,8 @@ void MainWindow::onSettingsClicked()
 
   DialogSettings dialog(this);
   dialog.exec();
-  bool previewPositionChanged = (_previewPosition != DialogSettings::previewPosition());
-  setPreviewPosition(DialogSettings::previewPosition());
+  bool previewPositionChanged = (_previewPosition != Settings::previewPosition());
+  setPreviewPosition(Settings::previewPosition());
   if (previewPositionChanged) {
     splitterSizes.clear();
     if (_previewPosition == PreviewPosition::Left) {
@@ -1382,7 +1394,7 @@ void MainWindow::onSettingsClicked()
     ui->splitter->setSizes(splitterSizes);
   }
   bool shouldUpdatePreview = false;
-  if (DialogSettings::logosAreVisible()) {
+  if (Settings::visibleLogos()) {
     if (!ui->logosLabel->isVisible()) {
       shouldUpdatePreview = true;
       ui->logosLabel->show();
@@ -1398,7 +1410,7 @@ void MainWindow::onSettingsClicked()
   }
   // Manage zoom constraints
   setZoomConstraint();
-  if (!DialogSettings::previewZoomAlwaysEnabled()) {
+  if (!Settings::previewZoomAlwaysEnabled()) {
     const FiltersPresenter::Filter & filter = _filtersPresenter->currentFilter();
     if (((ui->previewWidget->zoomConstraint() == ZoomConstraint::Fixed) && (ui->previewWidget->defaultZoomFactor() != ui->previewWidget->currentZoomFactor())) ||
         ((ui->previewWidget->zoomConstraint() == ZoomConstraint::OneOrMore) && (ui->previewWidget->currentZoomFactor() < 1.0))) {
